@@ -1,3 +1,15 @@
+#--------------------------------------------------------------------#
+# Crypt::RC4
+#       Date Written:   07-Jun-2000 04:15:55 PM
+#       Last Modified:  23-Nov-2001 01:20:31 PM
+#       Author:         Kurt Kincaid (sifukurt@yahoo.com)
+#       Copyright (c) 2001, Kurt Kincaid
+#       	All Rights Reserved.
+#
+#       This is free software and may be modified and/or
+#       redistributed under the same terms as Perl itself.
+#--------------------------------------------------------------------#
+
 package Crypt::RC4;
 
 use strict;
@@ -5,37 +17,46 @@ use vars qw($VERSION @ISA @EXPORT @EXPORT_OK);
 
 require Exporter;
 
-@ISA = qw(Exporter AutoLoader);
+@ISA = qw(Exporter);
 @EXPORT = qw(RC4);
-$VERSION = '1.11';
+$VERSION = '2.0';
+
+our ( $class, $key, @k, @s );
+
+sub new {
+    ( $class, $key )  = @_;
+    my $self = bless {}, $class;
+	Setup( $key );
+    return $self;
+}
 
 sub RC4 {
-	my $x = 0;
-	my $y = 0;
-
-	my $key = shift;
-	my @k = unpack( 'C*', $key );
-	my @s = 0..255;
-
-	for ($x = 0; $x != 256; $x++) {
-		$y = ( $k[$x % @k] + $s[$x] + $y ) % 256;
-		@s[$x, $y] = @s[$y, $x];
+	my ( $x, $y, $z );
+	if ( ref $_[0] ) {
+		my $self = shift;
+	} else {
+		Setup( shift );
 	}
-
-	$x = $y = 0;
-
-	my $z = undef;
-
 	for ( unpack( 'C*', shift ) ) {
 		$x = ($x + 1) % 256;
 		$y = ( $s[$x] + $y ) % 256;
 		@s[$x, $y] = @s[$y, $x];
 		$z .= pack ( 'C', $_ ^= $s[( $s[$x] + $s[$y] ) % 256] );
 	}
-
 	return $z;
 }
 
+sub Setup {
+    my ( $x, $y );
+	@k = unpack( 'C*', shift );
+	@s = 0..255;
+	for ($x = 0; $x != 256; $x++) {
+		$y = ( $k[$x % @k] + $s[$x] + $y ) % 256;
+		@s[$x, $y] = @s[$y, $x];
+	}
+}
+
+
 1;
 __END__
 
@@ -45,9 +66,19 @@ Crypt::RC4 - Perl implementation of the RC4 encryption algorithm
 
 =head1 SYNOPSIS
 
+# Functional Style
+
   use Crypt::RC4;
   $encrypted = RC4( $passphrase, $plaintext );
   $decrypt = RC4( $passphrase, $encrypted );
+  
+# OO Style
+  use Crypt::RC4;
+  $ref = Crypt::RC4->new( $passphrase );
+  $encrypted = $ref->RC4( $plaintext );
+  
+  $ref2 = Crypt::RC4->new( $passphrase );
+  $decrypted = $ref->RC4( $encrypted );
 
 =head1 DESCRIPTION
 
@@ -73,6 +104,6 @@ David Hook (dgh@wumpus.com.au)
 
 =head1 SEE ALSO
 
-perl(1), http://www.cypherspace.org, http://www.rsasecurity.com
+L<perl>, L<http://www.cypherspace.org>, L<http://www.rsasecurity.com>
 
 =cut
